package symantec.itools.awt.util;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;

/**
 * BeanInfo for Calendar.
 *
 */

public class CalendarBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a CalendarBeanInfo object.
	 */
	public CalendarBeanInfo() {
	}

	/**
	 * Gets a BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
            BeanInfo[] bi = new BeanInfo[1];
            bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
            return bi;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
		ResourceBundle group = ResourceBundle.getBundle("symantec.itools.resources.GroupBundle");
		String s=group.getString("GroupUtility"); 

		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setCanAddChild(false);
		bd.setWinHelp("0x1238C");
		bd.setFolder(s);
		bd.setToolbar(s);

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("CalendarC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("CalendarC32.gif");
			return img;
		}

		return null;
	}

	/**
	 * Gets an array of descriptions of the methods used for "connections" by
	 * Visual Caf's Interaction Wizard.
	 * Included in each method description is a CONNECTIONS ConnectionDescriptor.
	 * @return method descriptions for this bean
	 * @see symantec.itools.beans.ConnectionDescriptor#CONNECTIONS
	 */
	public MethodDescriptor[] getMethodDescriptors() {
		Class[] args;
		ConnectionDescriptor connection;
		java.util.Vector connections;
		java.util.Vector md = new java.util.Vector();
		ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");

		try{
			args = new Class[1];
			args[0] = java.lang.String.class ;
			MethodDescriptor setDate = new MethodDescriptor(beanClass.getMethod("setDate", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "String", "",
									"%name%.setDate(%arg%);",
									conn.getString("setDate"));
			connections.addElement(connection);

			setDate.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setDate);
		} catch (Exception e) { throw new Error("setDate:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getSelectedColor = new MethodDescriptor(beanClass.getMethod("getSelectedColor", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "Color", "",
									"%name%.getSelectedColor()",
									conn.getString("getSelectedColor"));
			connections.addElement(connection);

			getSelectedColor.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getSelectedColor);
		} catch (Exception e) { throw new Error("getSelectedColor:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.awt.Color.class ;
			MethodDescriptor setSelectedColor = new MethodDescriptor(beanClass.getMethod("setSelectedColor", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "Color", "",
									"%name%.setSelectedColor(%arg%);",
									conn.getString("setSelectedColor"));
			connections.addElement(connection);

			setSelectedColor.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setSelectedColor);
		} catch (Exception e) { throw new Error("setSelectedColor:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getDate = new MethodDescriptor(beanClass.getMethod("getDate", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "String", "",
									"%name%.getDate()",
									conn.getString("getDate"));
			connections.addElement(connection);

			getDate.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getDate);
		} catch (Exception e) { throw new Error("getDate:: " + e.toString()); }

		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		md.copyInto(rv);

		return rv;
	}

	/**
	 * Returns descriptions of this bean's properties.
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.resources.PropBundle");

		try{
		PropertyDescriptor defProperty = new PropertyDescriptor("date", beanClass);
		defProperty.setBound(true);
		defProperty.setConstrained(true);
		defProperty.setDisplayName(prop.getString("date"));

		PropertyDescriptor selectedColor = new PropertyDescriptor("selectedColor", beanClass);
		selectedColor.setBound(true);
		selectedColor.setConstrained(true);
		selectedColor.setDisplayName(prop.getString("selectedColor"));

		PropertyDescriptor layout = new PropertyDescriptor("layout", beanClass);
		layout.setHidden(true);

		PropertyDescriptor[] rv = {
			defProperty,
			selectedColor,
			layout};
		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	/**
	 * Returns the index of the property expected to be changed most often by the designer.
	 */
	public int getDefaultPropertyIndex() {
		return 0;	//  the index for our default property is always 0
	}

	private final static Class beanClass = Calendar.class;

	}	//  end of class CalendarBeanInfo