package symantec.itools.multimedia;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;

/**
 * BeanInfo for Firework.
 *
 */

public class FireworkBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a FireworkBeanInfo object.
	 */
	public FireworkBeanInfo() {
	}

	/**
	 * Gets a BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
            BeanInfo[] bi = new BeanInfo[1];
            bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
            return bi;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
		ResourceBundle group = ResourceBundle.getBundle("symantec.itools.resources.GroupBundle");
		String s=group.getString("GroupMultimedia"); 

		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setFolder(s);
		bd.setToolbar(s);
        bd.setWinHelp("0x12340");

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("FireworkC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("FireworkC32.gif");
			return img;
		}

		return null;
	}

	/**
	 * Gets an array of descriptions of the methods used for "connections" by
	 * Visual Caf's Interaction Wizard.
	 * Included in each method description is a CONNECTIONS ConnectionDescriptor.
	 * @return method descriptions for this bean
	 * @see symantec.itools.beans.ConnectionDescriptor#CONNECTIONS
	 */
	public MethodDescriptor[] getMethodDescriptors() {
		Class[] args;
		ConnectionDescriptor connection;
		java.util.Vector connections;
		java.util.Vector md = new java.util.Vector();
		ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");

		try{
			args = null;
			MethodDescriptor unfreezeRockets = new MethodDescriptor(beanClass.getMethod("unfreezeRockets", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.unfreezeRockets();",
									conn.getString("unfreezeRockets"));
			connections.addElement(connection);

			unfreezeRockets.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(unfreezeRockets);
		} catch (Exception e) { throw new Error("unfreezeRockets:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor freezeRockets = new MethodDescriptor(beanClass.getMethod("freezeRockets", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.freezeRockets();",
									conn.getString("freezeRockets"));
			connections.addElement(connection);

			freezeRockets.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(freezeRockets);
		} catch (Exception e) { throw new Error("freezeRockets:: " + e.toString()); }

		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		md.copyInto(rv);

		return rv;
	}

	public PropertyDescriptor[] getPropertyDescriptors() { return new PropertyDescriptor[0]; }

	private final static Class beanClass = Firework.class;

	}	//  end of class FireworkBeanInfo