package symantec.itools.awt;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;

//  09/07/97	LAB	Fixed misspelling of descriptions.
//  10/08/97	LAB	Added interaction for setFillMask (Addresses Mac Bug #8517).
//					Removed EditFont interactions and property.
//  10/11/97	LAB	Removed setFillMask interaction because it was redundant with base
//					classes set text to empty string interaction (Ooops. Addresses Mac
//					Bug #9133)

/**
 * BeanInfo for FormattedTextField.
 *
 */
public class FormattedTextFieldBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a FormattedTextFieldBeanInfo object.
	 */
	public FormattedTextFieldBeanInfo() {
	}

	/**
	 * Gets a BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
            BeanInfo[] bi = new BeanInfo[1];
            bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
            return bi;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
        ResourceBundle group = ResourceBundle.getBundle("symantec.itools.resources.GroupBundle");
		String s=group.getString("GroupAdditional"); 

		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setFolder(s);
		bd.setToolbar(s);
        bd.setWinHelp("0x123AE");

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("FormattedTextFieldC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("FormattedTextFieldC32.gif");
			return img;
		}

		return null;
	}

	/**
	 * Gets an array of descriptions of the methods used for "connections" by
	 * Visual Caf's Interaction Wizard.
	 * Included in each method description is a CONNECTIONS ConnectionDescriptor.
	 * @return method descriptions for this bean
	 * @see symantec.itools.beans.ConnectionDescriptor#CONNECTIONS
	 */
	public MethodDescriptor[] getMethodDescriptors() {
		Class[] args;
		ConnectionDescriptor connection;
		java.util.Vector connections;
		java.util.Vector md = new java.util.Vector();
		ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");
		String s;

		try{
			args = null;
			MethodDescriptor getMask = new MethodDescriptor(beanClass.getMethod("getMask", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "String", "",
									"%name%.getMask()",
									conn.getString("getMask"));
			connections.addElement(connection);

			getMask.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getMask);
		} catch (Exception e) { throw new Error("getMask:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.String.class ;
			MethodDescriptor setMask = new MethodDescriptor(beanClass.getMethod("setMask", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "String", "",
									"%name%.setMask(%arg%);",
									conn.getString("setMask"));
			connections.addElement(connection);

			setMask.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setMask);
		} catch (Exception e) { throw new Error("setMask:: " + e.toString()); }

		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		md.copyInto(rv);

		return rv;
	}

	/**
	 * Returns descriptions of this bean's properties.
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.resources.PropBundle");
		String s;

		try{
		PropertyDescriptor mask = new PropertyDescriptor("mask", beanClass);
		mask.setBound(true);
		mask.setConstrained(false);
		mask.setDisplayName(prop.getString("mask"));

		PropertyDescriptor[] rv = {
			mask
			};
		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	private final static Class beanClass = FormattedTextField.class;

	}	//  end of class FormattedTextFieldBeanInfo