package symantec.itools.awt;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;

//  08/09/97	LAB	Removed ListBoxFont property and connections.  Removed DisabledTextColor
//					property and connections.  Removed DefaultEnabledTextColor property and
//					connections.
//  09/04/97	LAB	Removed extranious semicolon in isEdited connection (Addresses Mac Bug #7806),
//					and fixed argument list to be null (it's a getter).
//  09/07/97	LAB	Fixed misspelling of descriptions.
//  09/08/97	LAB	Fixed method descriptor for isEdited connection.  Fixes introspection problem.

/**
 * BeanInfo for ImageListBox.
 *
 */
public class ImageListBoxBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a ImageListBoxBeanInfo object.
	 */
	public ImageListBoxBeanInfo() {
	}

	/**
	 * Gets a BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
            BeanInfo[] bi = new BeanInfo[1];
            bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
            return bi;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
        ResourceBundle group = ResourceBundle.getBundle("symantec.itools.resources.GroupBundle");
		String s=group.getString("GroupAdditional"); 

		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setCanAddChild(false);
		bd.setWinHelp("0x1238F");
		bd.setFolder(s);
		bd.setToolbar(s);

        ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");
		s=conn.getString("BORDER_REGULAR"); 
		bd.addConnectionDescriptor(new ConnectionDescriptor("output", "int", "",
												"%name%.BORDER_REGULAR",s));

		s=conn.getString("BORDER_NONE"); 
		bd.addConnectionDescriptor(new ConnectionDescriptor("output", "int", "",
												"%name%.BORDER_NONE",s));


		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("ImageListBoxC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("ImageListBoxC32.gif");
			return img;
		}

		return null;
	}

	/**
	 * Gets an array of descriptions of the methods used for "connections" by
	 * Visual Caf's Interaction Wizard.
	 * Included in each method description is a CONNECTIONS ConnectionDescriptor.
	 * @return method descriptions for this bean
	 * @see symantec.itools.beans.ConnectionDescriptor#CONNECTIONS
	 */
	public MethodDescriptor[] getMethodDescriptors() {
		Class[] args;
		ConnectionDescriptor connection;
		java.util.Vector connections;
		java.util.Vector md = new java.util.Vector();
		ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");

		try{
			args = null;
			MethodDescriptor getBorderType = new MethodDescriptor(beanClass.getMethod("getBorderType", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.getBorderType()",
									conn.getString("getBorderType"));
			connections.addElement(connection);

			getBorderType.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getBorderType);
		} catch (Exception e) { throw new Error("getBorderType:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.String.class ;
			MethodDescriptor addItem = new MethodDescriptor(beanClass.getMethod("addItem", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "String", "",
									"%name%.addItem(%arg%);",
									conn.getString("addItem"));
			connections.addElement(connection);

			addItem.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(addItem);
		} catch (Exception e) { throw new Error("addItem:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor selectAll = new MethodDescriptor(beanClass.getMethod("selectAll", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.selectAll();",
									conn.getString("selectAll"));
			connections.addElement(connection);

			selectAll.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(selectAll);
		} catch (Exception e) { throw new Error("selectAll:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getVisibleIndex = new MethodDescriptor(beanClass.getMethod("getVisibleIndex", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.getVisibleIndex()",
									conn.getString("getVisibleIndex"));
			connections.addElement(connection);

			getVisibleIndex.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getVisibleIndex);
		} catch (Exception e) { throw new Error("getVisibleIndex:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE ;
			MethodDescriptor setVisibleIndex = new MethodDescriptor(beanClass.getMethod("setVisibleIndex", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setVisibleIndex(%arg%);",
									conn.getString("setVisibleIndex"));
			connections.addElement(connection);

			setVisibleIndex.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setVisibleIndex);
		} catch (Exception e) { throw new Error("setVisibleIndex:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Boolean.TYPE ;
			MethodDescriptor setComboMode = new MethodDescriptor(beanClass.getMethod("setComboMode", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "boolean", "",
									"%name%.setComboMode(%arg%);",
									conn.getString("setComboMode"));
			connections.addElement(connection);

			setComboMode.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setComboMode);
		} catch (Exception e) { throw new Error("setComboMode:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor isMultipleSelections = new MethodDescriptor(beanClass.getMethod("isMultipleSelections", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "boolean", "",
									"%name%.isMultipleSelections()",
									conn.getString("allowMultipleSelections"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("output", "boolean", "",
									"!%name%.isMultipleSelections()",
									conn.getString("notMultipleSelections"));
			connections.addElement(connection);

			isMultipleSelections.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(isMultipleSelections);
		} catch (Exception e) { throw new Error("isMultipleSelections:: " + e.toString()); }

		try{
			args = new Class[2];
			args[0] = java.lang.Integer.TYPE ;
			args[1] = java.lang.Boolean.TYPE ;
			MethodDescriptor setSelectedIndex = new MethodDescriptor(beanClass.getMethod("setSelectedIndex", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setSelectedIndex(%arg%, true);",
									conn.getString("setSelectedIndex"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setSelectedIndex(%arg%, false);",
									conn.getString("resetSelectedIndex"));
			connections.addElement(connection);

			setSelectedIndex.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setSelectedIndex);
		} catch (Exception e) { throw new Error("setSelectedIndex:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor isShowVerticalScroll = new MethodDescriptor(beanClass.getMethod("isShowVerticalScroll", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "boolean", "",
									"%name%.isShowVerticalScroll()",
									conn.getString("visibleShowVerticalScroll"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("output", "boolean", "",
									"!%name%.isShowVerticalScroll()",
									conn.getString("hiddenShowVerticalScroll"));
			connections.addElement(connection);

			isShowVerticalScroll.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(isShowVerticalScroll);
		} catch (Exception e) { throw new Error("isShowVerticalScroll:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Boolean.TYPE ;
			MethodDescriptor setCellBorders = new MethodDescriptor(beanClass.getMethod("setCellBorders", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "boolean", "",
									"%name%.setCellBorders(%arg%);",
									conn.getString("setCellBordersValue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setCellBorders(true);",
									conn.getString("setCellBordersTrue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setCellBorders(false);",
									conn.getString("setCellBordersFalse"));
			connections.addElement(connection);

			setCellBorders.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setCellBorders);
		} catch (Exception e) { throw new Error("setCellBorders:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Boolean.TYPE ;
			MethodDescriptor setMultipleSelections = new MethodDescriptor(beanClass.getMethod("setMultipleSelections", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "boolean", "",
									"%name%.setMultipleSelections(%arg%);",
									conn.getString("setMultipleSelectionsValue"));
			connections.addElement(connection);

			setMultipleSelections.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setMultipleSelections);
		} catch (Exception e) { throw new Error("setMultipleSelections:: " + e.toString()); }

		try{
			args = new Class[2];
			args[0] = java.lang.String.class ;
			args[1] = java.lang.Boolean.TYPE ;
			MethodDescriptor setSelectedItem = new MethodDescriptor(beanClass.getMethod("setSelectedItem", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "String", "",
									"%name%.setSelectedItem(%arg%, true);",
									conn.getString("setSelectedItemString"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "String", "",
									"%name%.setSelectedItem(%arg%, false);",
									conn.getString("setSelectedItemFalse"));
			connections.addElement(connection);

			setSelectedItem.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setSelectedItem);
		} catch (Exception e) { throw new Error("setSelectedItem:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor isShowHorizontalScroll = new MethodDescriptor(beanClass.getMethod("isShowHorizontalScroll", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "boolean", "",
									"%name%.isShowHorizontalScroll()",
									conn.getString("visibleShowHorizontalScroll"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("output", "boolean", "",
									"!%name%.isShowHorizontalScroll()",
									conn.getString("hiddenShowHorizontalScroll"));
			connections.addElement(connection);

			isShowHorizontalScroll.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(isShowHorizontalScroll);
		} catch (Exception e) { throw new Error("isShowHorizontalScroll:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE ;
			MethodDescriptor delItem = new MethodDescriptor(beanClass.getMethod("delItem", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.delItem(%arg%);",
									conn.getString("delItem"));
			connections.addElement(connection);

			delItem.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(delItem);
		} catch (Exception e) { throw new Error("delItem:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor deselectAll = new MethodDescriptor(beanClass.getMethod("deselectAll", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.deselectAll();",
									conn.getString("deselectAll"));
			connections.addElement(connection);

			deselectAll.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(deselectAll);
		} catch (Exception e) { throw new Error("deselectAll:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Boolean.TYPE ;
			MethodDescriptor setShowVerticalScroll = new MethodDescriptor(beanClass.getMethod("setShowVerticalScroll", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "boolean", "",
									"%name%.setShowVerticalScroll(%arg%);",
									conn.getString("setShowVerticalScrollValue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setShowVerticalScroll(true);",
									conn.getString("setShowVerticalScrollTrue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setShowVerticalScroll(false);",
									conn.getString("setShowVerticalScrollFalse"));
			connections.addElement(connection);

			setShowVerticalScroll.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setShowVerticalScroll);
		} catch (Exception e) { throw new Error("setShowVerticalScroll:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE ;
			MethodDescriptor setRowsToShow = new MethodDescriptor(beanClass.getMethod("setRowsToShow", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setRowsToShow(%arg%);",
									conn.getString("setRowsToShow"));
			connections.addElement(connection);

			setRowsToShow.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setRowsToShow);
		} catch (Exception e) { throw new Error("setRowsToShow:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE ;
			MethodDescriptor isEdited = new MethodDescriptor(beanClass.getMethod("isEdited", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "boolean", "",
									"%name%.isEdited(%name%.getSelectedIndex())",
									conn.getString("isEdited"));
			connections.addElement(connection);

			isEdited.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(isEdited);
		} catch (Exception e) { throw new Error("isEdited:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getRows = new MethodDescriptor(beanClass.getMethod("getRows", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.getRows()",
									conn.getString("getRows"));
			connections.addElement(connection);

			getRows.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getRows);
		} catch (Exception e) { throw new Error("getRows:: " + e.toString()); }


		try{
			args = null;
			MethodDescriptor isCellBorders = new MethodDescriptor(beanClass.getMethod("isCellBorders", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "boolean", "",
									"%name%.isCellBorders()",
									conn.getString("isCellBordersOn"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("output", "boolean", "",
									"!%name%.isCellBorders()",
									conn.getString("isCellBordersOff"));
			connections.addElement(connection);

			isCellBorders.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(isCellBorders);
		} catch (Exception e) { throw new Error("isCellBorders:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE ;
			MethodDescriptor setBorderType = new MethodDescriptor(beanClass.getMethod("setBorderType", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setBorderType(%arg%);",
									conn.getString("setBorderType"));
			connections.addElement(connection);

			setBorderType.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setBorderType);
		} catch (Exception e) { throw new Error("setBorderType:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getLabel = new MethodDescriptor(beanClass.getMethod("getLabel", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "String", "",
									"%name%.getLabel()",
									conn.getString("getLabelILB"));
			connections.addElement(connection);

			getLabel.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getLabel);
		} catch (Exception e) { throw new Error("getLabel:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getSelectedItem = new MethodDescriptor(beanClass.getMethod("getSelectedItem", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "String", "",
									"%name%.getSelectedItem()",
									conn.getString("getSelectedItem"));
			connections.addElement(connection);

			getSelectedItem.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getSelectedItem);
		} catch (Exception e) { throw new Error("getSelectedItem:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor isAllSelected = new MethodDescriptor(beanClass.getMethod("isAllSelected", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "boolean", "",
									"%name%.isAllSelected()",
									conn.getString("isAllSelected"));
			connections.addElement(connection);

			isAllSelected.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(isAllSelected);
		} catch (Exception e) { throw new Error("isAllSelected:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor countItems = new MethodDescriptor(beanClass.getMethod("countItems", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.countItems()",
									conn.getString("countItems"));
			connections.addElement(connection);

			countItems.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(countItems);
		} catch (Exception e) { throw new Error("countItems:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor isComboMode = new MethodDescriptor(beanClass.getMethod("isComboMode", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "boolean", "",
									"%name%.isComboMode()",
									conn.getString("isComboMode"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("output", "boolean", "",
									"!%name%.isComboMode()",
									conn.getString("isMultiColMode"));
			connections.addElement(connection);

			isComboMode.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(isComboMode);
		} catch (Exception e) { throw new Error("isComboMode:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor clear = new MethodDescriptor(beanClass.getMethod("clear", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.clear();",
									conn.getString("clear"));
			connections.addElement(connection);

			clear.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(clear);
		} catch (Exception e) { throw new Error("clear:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Boolean.TYPE ;
			MethodDescriptor setShowHorizontalScroll = new MethodDescriptor(beanClass.getMethod("setShowHorizontalScroll", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "boolean", "",
									"%name%.setShowHorizontalScroll(%arg%);",
									conn.getString("setShowHorizontalScrollValue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setShowHorizontalScroll(true);",
									conn.getString("setShowHorizontalScrollTrue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setShowHorizontalScroll(false);",
									conn.getString("setShowHorizontalScrollFalse"));
			connections.addElement(connection);

			setShowHorizontalScroll.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setShowHorizontalScroll);
		} catch (Exception e) { throw new Error("setShowHorizontalScroll:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor delSelectedItems = new MethodDescriptor(beanClass.getMethod("delSelectedItems", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.delSelectedItems();",
									conn.getString("delSelectedItems"));
			connections.addElement(connection);

			delSelectedItems.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(delSelectedItems);
		} catch (Exception e) { throw new Error("delSelectedItems:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Boolean.TYPE ;
			MethodDescriptor setMultiColumnMode = new MethodDescriptor(beanClass.getMethod("setMultiColumnMode", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setMultiColumnMode(true);",
									conn.getString("setMultiColumnModeTrue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "void", "",
									"%name%.setMultiColumnMode(false);",
									conn.getString("setMultiColumnModeFalse"));
			connections.addElement(connection);

			setMultiColumnMode.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setMultiColumnMode);
		} catch (Exception e) { throw new Error("setMultiColumnMode:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getRowsToShow = new MethodDescriptor(beanClass.getMethod("getRowsToShow", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.getRowsToShow()",
									conn.getString("getRowsToShow"));
			connections.addElement(connection);

			getRowsToShow.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getRowsToShow);
		} catch (Exception e) { throw new Error("getRowsToShow:: " + e.toString()); }

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE ;
			MethodDescriptor setTopRow = new MethodDescriptor(beanClass.getMethod("setTopRow", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setTopRow(%arg%);",
									conn.getString("setTopRow"));
			connections.addElement(connection);

			setTopRow.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setTopRow);
		} catch (Exception e) { throw new Error("setTopRow:: " + e.toString()); }

		try{
			args = new Class[2];
			args[0] = java.lang.Integer.TYPE ;
			args[1] = java.lang.Boolean.TYPE ;
			MethodDescriptor setEnabled = new MethodDescriptor(beanClass.getMethod("setEnabled", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setEnabled(%arg%, true);",
									conn.getString("setEnabledTrue"));
			connections.addElement(connection);

			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setEnabled(%arg%, false);",
									conn.getString("setEnabledFalse"));
			connections.addElement(connection);

			setEnabled.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setEnabled);
		} catch (Exception e) { throw new Error("setEnabled:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getSelectedIndex = new MethodDescriptor(beanClass.getMethod("getSelectedIndex", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.getSelectedIndex()",
									conn.getString("getSelectedIndex"));
			connections.addElement(connection);

			getSelectedIndex.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getSelectedIndex);
		} catch (Exception e) { throw new Error("getSelectedIndex:: " + e.toString()); }

		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		md.copyInto(rv);

		return rv;
	}

	/**
	 * Returns descriptions of this bean's properties.
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.resources.PropBundle");

		try{
		PropertyDescriptor borderType = new PropertyDescriptor("borderType", beanClass);
		borderType.setBound(true);
		borderType.setConstrained(true);
		borderType.setDisplayName(prop.getString("borderType"));
		borderType.setValue("ENUMERATION","BORDER_REGULAR=0, BORDER_NONE=1");

		PropertyDescriptor cellBorders = new PropertyDescriptor("cellBorders", beanClass);
		cellBorders.setBound(true);
		cellBorders.setConstrained(true);
		cellBorders.setDisplayName(prop.getString("cellBorders"));

		PropertyDescriptor columns = new PropertyDescriptor("columns", beanClass);
		columns.setBound(true);
		columns.setConstrained(true);
		columns.setDisplayName(prop.getString("columns"));

		PropertyDescriptor comboMode = new PropertyDescriptor("comboMode", beanClass);
		comboMode.setBound(true);
		comboMode.setConstrained(true);
		comboMode.setDisplayName(prop.getString("comboMode"));

		PropertyDescriptor listItems = new PropertyDescriptor("listItems", beanClass);
		listItems.setBound(true);
		listItems.setConstrained(true);
		listItems.setDisplayName(prop.getString("listItems"));

		PropertyDescriptor multiColumnMode = new PropertyDescriptor("multiColumnMode", beanClass, null, "setMultiColumnMode");
		multiColumnMode.setBound(true);
		multiColumnMode.setConstrained(true);
		multiColumnMode.setDisplayName(prop.getString("multiColumnMode"));

		PropertyDescriptor multipleSelections = new PropertyDescriptor("multipleSelections", beanClass);
		multipleSelections.setBound(true);
		multipleSelections.setConstrained(true);
		multipleSelections.setDisplayName(prop.getString("multipleSelections"));

		PropertyDescriptor rowsToShow = new PropertyDescriptor("rowsToShow", beanClass);
		rowsToShow.setBound(true);
		rowsToShow.setConstrained(true);
		rowsToShow.setDisplayName(prop.getString("rowsToShow"));

		PropertyDescriptor showHorizontalScroll = new PropertyDescriptor("showHorizontalScroll", beanClass);
		showHorizontalScroll.setBound(true);
		showHorizontalScroll.setConstrained(true);
		showHorizontalScroll.setDisplayName(prop.getString("showHorizontalScroll"));

		PropertyDescriptor showVerticalScroll = new PropertyDescriptor("showVerticalScroll", beanClass);
		showVerticalScroll.setBound(true);
		showVerticalScroll.setConstrained(true);
		showVerticalScroll.setDisplayName(prop.getString("showVerticalScroll"));

		PropertyDescriptor layout = new PropertyDescriptor("layout", beanClass);
		layout.setHidden(true);

		PropertyDescriptor[] rv = {
			borderType,
			cellBorders,
			columns,
			comboMode,
			listItems,
			multiColumnMode,
			multipleSelections,
			rowsToShow,
			showHorizontalScroll,
			showVerticalScroll,
			layout};
		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	private final static Class beanClass = ImageListBox.class;

	}	//  end of class ImageListBoxBeanInfo