package symantec.itools.awt.util.spinner;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;

//  09/11/97	LAB	Removed redundant orientation and editable properties and connections
//					(Addresses Mac Bug #7820).

/**
 * BeanInfo for ListSpinner.
 *
 */
public class ListSpinnerBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a ListSpinnerBeanInfo object.
	 */
	public ListSpinnerBeanInfo() {
	}

	/**
	 * Gets a BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
            BeanInfo[] bi = new BeanInfo[1];
            bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
            return bi;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
		ResourceBundle group = ResourceBundle.getBundle("symantec.itools.resources.GroupBundle");
		String s=group.getString("GroupAdditional"); 

		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setCanAddChild(false);
		bd.setFolder(s);
		bd.setToolbar(s);
        bd.setWinHelp("0x123A4");

		bd.addAdditionalConnections(getAdditionalBeanInfo());

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("ListSpinnerC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("ListSpinnerC32.gif");
			return img;
		}

		return null;
	}

	/**
	 * Gets an array of descriptions of the methods used for "connections" by
	 * Visual Caf's Interaction Wizard.
	 * Included in each method description is a CONNECTIONS ConnectionDescriptor.
	 * @return method descriptions for this bean
	 * @see symantec.itools.beans.ConnectionDescriptor#CONNECTIONS
	 */
	public MethodDescriptor[] getMethodDescriptors() {
		Class[] args;
		ConnectionDescriptor connection;
		java.util.Vector connections;
		java.util.Vector md = new java.util.Vector();
		ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");

		try{
			args = new Class[1];
			args[0] = java.lang.String.class ;
			MethodDescriptor addItem = new MethodDescriptor(beanClass.getMethod("addItem", args));
			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "String", "",
									"%name%.addItem(%arg%);",
									conn.getString("addItemSpinner"));
			connections.addElement(connection);

			addItem.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(addItem);
		} catch (Exception e) { throw new Error("addItem:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getCurrentText = new MethodDescriptor(beanClass.getMethod("getCurrentText", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "String", "",
									"%name%.getCurrentText()",
									conn.getString("getCurrentText"));
			connections.addElement(connection);

			getCurrentText.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getCurrentText);
		} catch (Exception e) { throw new Error("getCurrentText:: " + e.toString()); }

		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		md.copyInto(rv);

		return rv;
	}

	/**
	 * Returns descriptions of this bean's properties.
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.resources.PropBundle");

		try{
		PropertyDescriptor listItems = new PropertyDescriptor("listItems", beanClass);
		listItems.setBound(true);
		listItems.setConstrained(true);
		listItems.setDisplayName(prop.getString("listItems"));

		PropertyDescriptor allowDynamicResizing = new PropertyDescriptor("allowDynamicResizing", beanClass);
		allowDynamicResizing.setBound(true);
		allowDynamicResizing.setConstrained(true);
		allowDynamicResizing.setDisplayName(prop.getString("allowDynamicResizing"));

		PropertyDescriptor[] rv = {
			listItems,
			allowDynamicResizing};
		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	private final static Class beanClass = ListSpinner.class;

	}	//  end of class ListSpinnerBeanInfo