package symantec.itools.awt;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;

/**
 * BeanInfo for TabPanel.
 *
 */

public class TabPanelBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a TabPanelBeanInfo object.
	 */
	public TabPanelBeanInfo() {
	}

	/**
	 * Gets a BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
            BeanInfo[] bi = new BeanInfo[1];
            bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
            return bi;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
        ResourceBundle group = ResourceBundle.getBundle("symantec.itools.resources.GroupBundle");
		String s=group.getString("GroupPanel"); 

		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setFolder(s);
		bd.setToolbar(s);
        bd.setWinHelp("0x123B2");

		bd.addAdditionalConnections(getAdditionalBeanInfo());

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("TabPanelC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("TabPanelC32.gif");
			return img;
		}

		return null;
	}

	/**
	 * Gets an array of descriptions of the methods used for "connections" by
	 * Visual Caf's Interaction Wizard.
	 * Included in each method description is a CONNECTIONS ConnectionDescriptor.
	 * @return method descriptions for this bean
	 * @see symantec.itools.beans.ConnectionDescriptor#CONNECTIONS
	 */
	public MethodDescriptor[] getMethodDescriptors() {
		Class[] args;
		ConnectionDescriptor connection;
		java.util.Vector connections;
		java.util.Vector md = new java.util.Vector();
		ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE ;
			MethodDescriptor setCurrentPanelNdx = new MethodDescriptor(beanClass.getMethod("setCurrentPanelNdx", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setCurrentPanelNdx(%arg%);",
									conn.getString("setCurrentPanelNdx"));
			connections.addElement(connection);

			setCurrentPanelNdx.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setCurrentPanelNdx);
		} catch (Exception e) { throw new Error("setCurrentPanelNdx:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getCurrentPanelNdx = new MethodDescriptor(beanClass.getMethod("getCurrentPanelNdx", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.getCurrentPanelNdx()",
									conn.getString("getCurrentPanelNdx"));
			connections.addElement(connection);

			getCurrentPanelNdx.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getCurrentPanelNdx);
		} catch (Exception e) { throw new Error("getCurrentPanelNdx:: " + e.toString()); }

		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		md.copyInto(rv);

		return rv;
	}

	/**
	 * Returns descriptions of this bean's properties.
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.resources.PropBundle");

		try{
		PropertyDescriptor defProperty = new PropertyDescriptor("currentPanelNdx", beanClass);
		defProperty.setBound(true);
		defProperty.setConstrained(true);
		defProperty.setDisplayName(prop.getString("currentPanelNdx"));

		PropertyDescriptor panelLabels = new PropertyDescriptor("panelLabels", beanClass);
		panelLabels.setBound(true);
		panelLabels.setConstrained(true);
		panelLabels.setDisplayName(prop.getString("panelLabels"));

		PropertyDescriptor tabsOnBottom = new PropertyDescriptor("tabsOnBottom", beanClass);
		tabsOnBottom.setBound(true);
		tabsOnBottom.setConstrained(true);
		tabsOnBottom.setDisplayName(prop.getString("tabsOnBottom"));

		PropertyDescriptor layout = new PropertyDescriptor("layout", beanClass);
		layout.setHidden(true);

		PropertyDescriptor[] rv = {
			defProperty,
			panelLabels,
			tabsOnBottom,
			layout};
		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	/**
	 * Returns the index of the property expected to be changed most often by the designer.
	 */
	public int getDefaultPropertyIndex() {
		return 0;	//  the index for our default property is always 0
	}

	private final static Class beanClass = TabPanel.class;

	}	//  end of class TabPanelBeanInfo