package symantec.itools.awt.util;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;

//  07/24/97	LAB	Added Space property and connections.

/**
 * BeanInfo for ToolBarSpacer.
 *
 */

public class ToolBarSpacerBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a ToolBarSpacerBeanInfo object.
	 */
	public ToolBarSpacerBeanInfo() {
	}

	/**
	 * Gets a BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
            BeanInfo[] bi = new BeanInfo[1];
            bi[0] = Introspector.getBeanInfo(beanClass.getSuperclass());
            return bi;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
		ResourceBundle group = ResourceBundle.getBundle("symantec.itools.resources.GroupBundle");
		String s=group.getString("GroupPanel"); 

		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setFolder(s);
		bd.setToolbar(s);
        bd.setWinHelp("0x123B3");

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("ToolBarSpacerC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("ToolBarSpacerC32.gif");
			return img;
		}

		return null;
	}

	/**
	 * Gets an array of descriptions of the methods used for "connections" by
	 * Visual Caf's Interaction Wizard.
	 * Included in each method description is a CONNECTIONS ConnectionDescriptor.
	 * @return method descriptions for this bean
	 * @see symantec.itools.beans.ConnectionDescriptor#CONNECTIONS
	 */
	public MethodDescriptor[] getMethodDescriptors() {
		Class[] args;
		ConnectionDescriptor connection;
		java.util.Vector connections;
		java.util.Vector md = new java.util.Vector();
		ResourceBundle conn = ResourceBundle.getBundle("symantec.itools.resources.ConnBundle");

		try{
			args = new Class[1];
			args[0] = java.lang.Integer.TYPE;
			MethodDescriptor setSpace = new MethodDescriptor(beanClass.getMethod("setSpace", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("input", "int", "",
									"%name%.setSpace(%arg%);",
									conn.getString("setSpace"));
			connections.addElement(connection);

			setSpace.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(setSpace);
		} catch (Exception e) { throw new Error("setSpace:: " + e.toString()); }

		try{
			args = null;
			MethodDescriptor getSpace = new MethodDescriptor(beanClass.getMethod("getSpace", args));

			connections = new java.util.Vector();
			connection = new ConnectionDescriptor("output", "int", "",
									"%name%.getSpace()",
									conn.getString("getSpace"));
			connections.addElement(connection);

			getSpace.setValue(ConnectionDescriptor.CONNECTIONS, connections);
			md.addElement(getSpace);
		} catch (Exception e) { throw new Error("getSpace:: " + e.toString()); }

		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		md.copyInto(rv);

		return rv;
	}

	/**
	 * Returns descriptions of this bean's properties.
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.resources.PropBundle");

		try{

		PropertyDescriptor space = new PropertyDescriptor("space", beanClass);
		space.setBound(true);
		space.setConstrained(true);
		space.setDisplayName(prop.getString("space"));

		PropertyDescriptor[] rv = {
			space};
		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	private final static Class beanClass = ToolBarSpacer.class;

	}	//  end of class ToolBarSpacerBeanInfo